/****************************************************************************
 Creation ??/??/1999                                               by Sylvain


                          JPEG2HTML.C
                          ------------


*****************************************************************************

  Generation d'une page html comprenant des liens sur les images
  du chemin passe en argument + creation des petites images dans
  le sous repertoire "smalls".
  
    Utilisation :   
    -------------
    jpeg2html /home/moi/mon_repertoire_plein_d_images > ma_page_woueb_a_moi.html
    

****************************************************************************/
#ifdef _WIN32
#include <windows.h>
#undef FAR
#include <io.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <setjmp.h>
#include <string.h>
#include "parcoursdir.h"
#include "dcpjpeg.h"
#include "jpeg2jpeg.h"
#include "createdirectorytree.h"

// #define pas definis ...
#ifndef MAX_PATH
#define MAX_PATH    1024
#endif


// Protos
static void GenerateHtmlPage( char * pBody, int * pcbBody, char * pszPath );
static BOOL FillHtmlTable( char * pszReper, char * pszFile, void * lParam);
static BOOL CreateSmallImage( char * pszReper, char * pszFile, int * pWidth, int *pHeight );


/*---------------------------------------------------------------------------
**
**        MAIN
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --main--
**  ARGUMENTS
**        Argument1:  int argc
**        Argument2: char * argv[]
**  RETOUR/RESULTAT
**        int 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 20/04/2001 
**--------------------------------------------------------------------------*/
int main( int argc, char * argv[] )
    {
    char szPath[MAX_PATH];
    char * pBody;
    int cbBody;
    int fd = 1;

    // Chemin des fichiers jpeg
    if( argc >= 2 )
        {        
        strcpy( szPath, argv[1] );
        if( szPath[strlen(szPath)-1] != '\\' || szPath[strlen(szPath)-1] != '/' )
            strcat( szPath, "/" );
        }
    else
        {
            strcpy( szPath, "./" );
        }

    // Fichier de sortie
    if( argc >= 3 )
        {
	fd = creat(argv[2], 0664);
        if( fd < 0 ) fd = 1;

#ifdef _WIN32
	/* Mode texte par defaut sous win32. On reste coherent et on
	   cree et ouvre le fichier en mode binaire */
	setmode( fd, O_BINARY );
#endif
        }
    
    // On prend (sans doute) pas de risque (200000)
    pBody = (char *)malloc( 200000 );
    GenerateHtmlPage( pBody, &cbBody, szPath );

    /*  Ecrit la page  */
    write( fd, pBody, cbBody );
    if( fd != 1 ) close( fd );
    return 0;
    }



/*---------------------------------------------------------------------------
**
**        GENERATEHTMLPAGE
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --GenerateHtmlPage--
**
**	Genere une page web avec des lien sur les images du repertoire
**	pszPath
**	La page est generee en memoire dans pBody
**
**
**  ARGUMENTS
**        Argument1:  char * pBody  [OUT]
**        Argument2: int * pcbBody  Pas utilise
**        Argument3: char * pszPath [IN]
**  RETOUR/RESULTAT
**        void 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 09/04/2001 
**--------------------------------------------------------------------------*/
void GenerateHtmlPage( char * pBody, int * pcbBody, char * pszPath )
    {
    char * p = pBody;

    /*  On initilise la page  */
    p += sprintf( p, "<html>\n" );

    p += sprintf( p, "<head>\n" );
    p += sprintf( p, "<meta http-equiv=\"Content-Type\"\n" );
    p += sprintf( p, "<content=\"text/html; charset=iso-8859-1\">\n" );
    p += sprintf( p, "<title></title>\n" );
    p += sprintf( p, "</head>\n" );

    p += sprintf( p, "<body>\n" );

    p += sprintf( p, "<p align=\"center\"><b><u>Images</u></b></p>\n" );
    
    p += sprintf( p, "<div align=\"center\"><center>\n" );
    p += sprintf( p, "<table border=\"0\" cellpadding=\"10\" cellspacing=\"10\">\n" );
    
    // On cree la page (FALSE = on ne parcourt que le repertoire passe
    // en argument, pas ses sous repertoires
    ParcoursDirectory2( pszPath, FALSE, FillHtmlTable, (void *)&p  );        

    p += sprintf( p, "</table>\n" );
    p += sprintf( p, "</center></div>\n" );

    p += sprintf( p, "</body>\n" );
    p += sprintf( p, "</html>\n" );

    *pcbBody = (p - pBody);
}


/*---------------------------------------------------------------------------
**
**        CREATESMALLIMAGE
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --CreateSmallImage--
**
**	Cree la petite image
**
**  ARGUMENTS
**        Argument1:  char * pszReper
**        Argument2: char * pszFile
**        Argument3: void * lParam
**  RETOUR/RESULTAT
**        BOOL FALSE erreur
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 09/04/2001 
**--------------------------------------------------------------------------*/
BOOL CreateSmallImage( char * pszReper, char * pszFile, int * pWidth, int *pHeight )
    {
    char szFileName[MAX_PATH];
    char szFileNameDest[MAX_PATH];
    
    // ouvrir le fichier jpg et trouver la hauteur pour une largeur de 100
    sprintf( szFileName, "%s%s", pszReper, pszFile );
    if( ! GetJpegSize(szFileName, pWidth, pHeight ) || (*pWidth == 0 || *pHeight == 0 ) )
        return FALSE;
    

    // generer une petite image (prefixee par p)
    // sprintf( szFileNameDest,"%s/smalls/s%s", pszReper, pszFile );
    sprintf( szFileNameDest,"%s/smalls/", pszReper );

    // On cree le repertoire small s'il n'existe pas
    CreateDirectoryTree( szFileNameDest );
    strcat( szFileNameDest,"s" );
    strcat( szFileNameDest,pszFile );

    // Creation image 100 pixel, qualite = 50
    Jpeg2Jpeg( szFileName, szFileNameDest, 100, 50 );
    return TRUE;
    }



/*---------------------------------------------------------------------------
**
**        FILLHTMLTABLE
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --FillHtmlTable--
**
**	Callback de ParcoursDir() appele pour chaque fichier de l'arborescence
**
**  ARGUMENTS
**        Argument1:  char * pszReper
**        Argument2: char * pszFile
**        Argument3: void * lParam
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 09/04/2001 
**--------------------------------------------------------------------------*/
BOOL FillHtmlTable( char * pszReper, char * pszFile, void * lParam)
    {
    static int i;
    char ** pp = (char **)lParam;
    char *p = *pp;
    int Width, Height; char szFileName[MAX_PATH];
    BOOL fAvancepp = TRUE;

    // Si c'est pas un jpeg, on se casse
    if( ! strstr(pszFile, ".jpg" ) && ! strstr(pszFile, ".jpeg" ) )
	return TRUE;

    // Debut de ligne
    if( (i % 4) == 0 )
        {
        p += sprintf( p, "<tr>\n" );
        }
 
    // Cree la petite image
    if( ! CreateSmallImage( pszReper, pszFile, &Width, &Height ) ) return TRUE;
	
    // lien sur image (le lien src est precede de p (image.jpg -> pimage.jpg, comme petite)
    p += sprintf( p, "<td align=\"center\">"
	"<a href=\"%s\" ><img src=\"smalls/s%s\" border=\"0\" width=\"%d\" height=\"%d\"></a>&nbsp; <br>&nbsp;</td>", 
	pszFile, pszFile, 100, 100 * Height / Width );
   
        
    // Fin de ligne
    if( (i % 4) == 3 )
        {
	p += sprintf( p, "</tr>\n" );
	}

    *pp = p;

    i ++;
    return TRUE;
    }



