/****************************************************************************
 Creation 06/04/2001                                               by Sylvain


                          PARCOURSDIR.C
                          -------------


*****************************************************************************

         Ce programme assure :

  Parcours recursif d'une arborescence pszReper avec execution du pointeur
  de fonction (*Traite)() passe en argument.

  Si (*Traite)( pszReper, pszFile, lParam) renvoie FALSE, le parcours s'arrete


****************************************************************************/
#include <stdio.h>

#ifdef _WIN32
#include <windows.h>
#else   /*_WIN32*/
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#endif  /*_WIN32*/

#include "parcoursdir.h"


/*---------------------------------------------------------------------------
**
**        PARCOURSDIRECTORYTREE
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --ParcoursDirectoryTree--
**
**	Traite chaque repertoire 1 seule fois.
**	RQ : Les repertoires crees par le callback seront traites
**
**  ARGUMENTS
**        Argument1:  char * pszReper
**        Argument2: (*TraiteTree)( char * pszReper,void * lParamTree)
**        Argument3: void * lParamTree
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 20/04/2001 
**--------------------------------------------------------------------------*/
BOOL ParcoursDirectoryTree( char * pszReper, 
			   BOOL (*TraiteTree)( char * pszReper, void * lParamTree), void * lParamTree  )
{
    return ParcoursDirectory3( pszReper, TRUE , NULL, NULL, TraiteTree, lParamTree );
}


/*---------------------------------------------------------------------------
**
**        PARCOURSDIRECTORY
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --ParcoursDirectory--
**  ARGUMENTS
**        Argument1:  char * pszReper
**        Argument2: (*Traite)( char * pszReper, char * pszFile, void * lParam)
**        Argument3: void * lParam
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 06/04/2001 
**--------------------------------------------------------------------------*/
BOOL ParcoursDirectory( char * pszReper, 
		       BOOL (*Traite)( char * pszReper, char * pszFile, void * lParam), void * lParam  )
{
    return ParcoursDirectory2( pszReper, TRUE, Traite, lParam  );
}

/*---------------------------------------------------------------------------
**
**        PARCOURSDIRECTORY2
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --ParcoursDirectory2--
**  ARGUMENTS
**        Argument1:  char * pszReper
**        Argument2:  BOOL fResurse	Si FALSE, ne traite que pszReper, pas ses sous rep.
**        Argument3: (*Traite)( char * pszReper, char * pszFile, void * lParam)
**        Argument4: void * lParam
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 06/04/2001 
**--------------------------------------------------------------------------*/
/**********************************************************/
BOOL ParcoursDirectory2( char * pszReper, BOOL fRecurse,
	BOOL (*Traite)( char * pszReper, char * pszFile, void * lParam), void * lParam  )
{
    return ParcoursDirectory3( pszReper, fRecurse,Traite, lParam, NULL, NULL  );

}

#ifdef _WIN32
BOOL ParcoursDirectory3( char * pszReper, BOOL fRecurse,
	BOOL (*Traite)( char * pszReper, char * pszFile, void * lParam), void * lParam,
	BOOL (*TraiteTree)( char * pszReper, void * lParamTree), void * lParamTree  )
{
    HANDLE hFindFile;
    WIN32_FIND_DATA FindFileData;
    BOOL fEnd;
    char szAllFileNames[MAX_PATH];
    char cFileName[MAX_PATH];
    
    if( TraiteTree )
    {
	if( ! TraiteTree( pszReper, lParamTree ) )
	{
	    fEnd = TRUE;
	}
    }

    /*  Premiere entree */
    strcpy( szAllFileNames, pszReper );
    if( pszReper[strlen(pszReper)-1] != '\\' )
        strcat( szAllFileNames, "\\" );
    strcat( szAllFileNames, "*.*" );
    hFindFile = FindFirstFile( szAllFileNames, &FindFileData );
    if( hFindFile == INVALID_HANDLE_VALUE )
        return FALSE;
    
    /*  On traite recursivement s'il s'agit de repertoire   */
    fEnd = FALSE;
    do  {    
        /*  Pas . et .. */
        if( strcmp( FindFileData.cFileName, "." ) == 0 ||
	    strcmp( FindFileData.cFileName, ".." ) == 0 )
            continue;
	
        /*  Repertoire ???  */
        strcpy( cFileName, pszReper );
        if( pszReper[strlen(pszReper)-1] != '\\' )
            strcat( cFileName, "\\" );
        strcat( cFileName, FindFileData.cFileName );
        if( FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
	{
	    if( ! fRecurse )
		continue;
            if( ! ParcoursDirectory3( cFileName, TRUE, Traite, lParam, TraiteTree, lParamTree ) )
	    {
                fEnd = TRUE;
	    }
	}
        
        else
            /*  Le traitement  a faire  pour le fichier */
	{
	    if( Traite )
	    {
		if( ! Traite( pszReper, FindFileData.cFileName, lParam ) )
		{
		    fEnd = TRUE;
		}
	    }
	}
        
        if( fEnd )
	{
            FindClose( hFindFile );    
            return FALSE;
	}
	
        /*  Tant que ca marche  */
    } while( FindNextFile( hFindFile, &FindFileData ) );
    
    FindClose( hFindFile );    
    return TRUE;
}

#else   /*_WIN32*/
#ifndef MAX_PATH
#define MAX_PATH    1024
#endif
BOOL ParcoursDirectory3( char * pszReper, BOOL fRecurse,
	BOOL (*Traite)( char * pszReper, char * pszFile, void * lParam), void * lParam,
	BOOL (*TraiteTree)( char * pszReper, void * lParamTree), void * lParamTree  )
{
    DIR * pDir;
    struct dirent * pFile;
    char cFileName[MAX_PATH];
    BOOL fEnd;
    
    if( TraiteTree )
    {
	if( ! TraiteTree( pszReper, lParamTree ) )
	{
	    fEnd = TRUE;
	}
    }
    /*  On met . au bout */
    sprintf( cFileName, "%s.", pszReper );
    /*printf( "\t%s\n", pszReper );*/
    if( (pDir = opendir(pszReper)) == NULL )
    {
	printf( "opendir(%s) == NULL\n", pszReper );
	return FALSE;
    }
    
    /*  opendir() ne positionne pas sur le 1 fichier ...*/
    fEnd = FALSE;
    while( (pFile = readdir(pDir)) != NULL )
    {
	char szCurrentFile[MAX_PATH];
	struct stat st;
        
        /*  Pas . et .. */
        if( strcmp( pFile->d_name, "." ) == 0 || strcmp( pFile->d_name, ".." ) == 0 )
            continue;
	
	/*  On concatene  */
	sprintf( szCurrentFile, "%s%s", pszReper, pFile->d_name );
        
	/*  Fichier ou repertoire ? */
	if (stat(szCurrentFile, &st) != 0)
	{
	    printf("stat() error %d (file '%s')\n", errno, szCurrentFile ) ;
	    continue ;
	}
	if( st.st_mode & S_IFDIR )
	    /*if( pFile->d_type & DT_DIR )*/
	{
	/*
	printf( "C'est un repertoire (%s)\n", szCurrentFile );
	    */
	    if( ! fRecurse )
		continue;
            sprintf( cFileName, "%s%s/", pszReper, pFile->d_name );
            if( ! ParcoursDirectory3( cFileName, TRUE, Traite, lParam, TraiteTree, lParamTree ) )
	    {
		fEnd = TRUE;
	    }
	}
        else
            /*  Le traitement  a faire  pour le fichier */
	{
	    if( Traite )
	    {
		if( ! Traite( pszReper, pFile->d_name, lParam ) )
		{
		    fEnd = TRUE;
		}
	    }
	}
        
        /*  On arrete de boucler et on se casse  */
        if( fEnd )
	{
            closedir( pDir );    
            return FALSE;
	}
    }
    
    /*  Restons propres  */
    closedir( pDir );    
    return TRUE;	
}
#endif  /*_WIN32*/


